#!/bin/bash


# Info: Noetige PyQt Version: PyQt5==5.10.0

# Xcode Commandline Tools:
# Install: xcode-select --install (da kommt Python 3.7 mit)
# Ort: /Library/Developer/CommandLineTools/
# pip3 --user Verzeichnis: /Users/pkloss/Library/Python/
# Achtung: pip3 user update zerstoert pip3

# Definition
PY_MODULE="folium,xlsxwriter,psycopg2,PyQt5,PyQt5.QtWebEngineWidgets,openpyxl,requests,psutil"
PY_MODULE_PIP="folium xlsxwriter psycopg2-binary PyQt5 PyQtWebEngine openpyxl requests psutil"
INFO_INST_MANUELL="\n
* Please install Python3.7 first:\nwww.python.org/downloads\n
* Then install the Xcode CommandLineTools the command line:\nxcode-select --install\n
* Then install the following modules in the command line:\npip3 install --user $PY_MODULE_PIP\n
* When you contact a CRITTERBASE administrator, please send him the console output that has been opened right now"
XCODE_COMMANDLINETOOLS="/Library/Developer/CommandLineTools"

# Infos
function info_critterbase(){
echo | tee -a install.log.txt
echo '   ___ ___ ___ _____ _____ ___ ___ ___   _   ___ ___ ' | tee -a install.log.txt
echo '  / __| _ \_ _|_   _|_   _| __| _ \ _ ) /_\ / __| __|' | tee -a install.log.txt
echo ' | (__|   /| |  | |   | | | _||   / _ \/ _ \\__ \ _| ' | tee -a install.log.txt
echo '  \___|_|_\___| |_|   |_| |___|_|_\___/_/ \_\___/___|' | tee -a install.log.txt
echo '                                                     ' | tee -a install.log.txt
echo '=====================================================' | tee -a install.log.txt
echo '                  Install/Repair Script' | tee -a install.log.txt
echo '=====================================================' | tee -a install.log.txt
}

function info_section(){
echo | tee -a install.log.txt
echo '-----------------------------------------------------' | tee -a install.log.txt
echo $* | tee -a install.log.txt
echo '-----------------------------------------------------' | tee -a install.log.txt
echo | tee -a install.log.txt
}

# Dialog
function dialog_info() {
    osascript -e 'display alert "'"$1"'" message "'"$2"'"'
}
function dialog_info_critical() {
    osascript -e 'display alert "'"$1"'" message "'"$2"'" as critical'
}

# Fehler
function fehler(){
	osascript -e 'display notification "installation failed ..." with title "CRITTERBASE" sound name "submarine"'
	open -a TextEdit install.log.txt
	dialog_info_critical "Automatic installation failed" "Please install the necessary dependencies manually. $INFO_INST_MANUELL"
    exit
}

# Teste Rueckgabewert
function check_prg(){
    eval "$*" >> install.log.txt 2>&1
    if test $? -ne 0; then RETURN=false; else RETURN=true; fi
}

# ----

# Log
rm install.log.txt 2> /dev/null
touch install.log.txt

# Pfade holen, damit Python3 gefunden wird
source /etc/profile 2> /dev/null
source ~/.bash_profile2 2> /dev/null

# Wechsel in Verzeichnis indem dieses Programm liegt
DIR="$(dirname "$0")"
cd $DIR

# Intro
clear
info_critterbase
RET=$(osascript -e 'display alert "Install or Repair\nCRITTERBASE Collector App" message "Press OK to install/repair all needed dependencies" buttons {"OK","Cancel"} default button 1')
if test "$RET" = "button returned:Cancel"; then exit; fi

# Teste xcode command line tools
info_section "Testing Xcode Command Line Tools"
echo "Location: "$XCODE_COMMANDLINETOOLS | tee -a install.log.txt
if test -d $XCODE_COMMANDLINETOOLS; then
  echo "Command Line Tools found" | tee -a install.log.txt
  RET=$(osascript -e 'display alert "Xcode Command Line Tools found" message "It seems, that Xcode Command Line Tools are installed" buttons {"OK","Cancel"} default button 1')
  if test "$RET" = "button returned:Cancel"; then exit; fi
else
  echo "Command Line Tools not found" | tee -a install.log.txt
  echo "* Starting installation" | tee -a install.log.txt
  xcode-select --install
  RET=$(osascript -e 'display alert "Installation of Xcode Command Line Tools" message "If a window with installation instructions has opened now, follow the instructions and click OK when all is done. Otherwise if no window has opened and the console output says that everything is installed already click OK" buttons {"OK","Cancel"} default button 1')
  if test "$RET" = "button returned:Cancel"; then exit; fi
fi

# Teste Python3
info_section "Testing Python3"
check_prg 'which python3'
if ! $RETURN; then
    # Python3 ist nicht installiert
    echo "Python3 is not installed" | tee -a install.log.txt
    RET=$(osascript -e 'display alert "Python3 is not installed" message "Please install Python3.7+ from www.python.org/downloads" buttons {"OK","Cancel"} default button 1')
    exit
fi
echo "Python3 is installed" | tee -a install.log.txt
echo "Version: " $(python3 -V) | tee -a install.log.txt
echo "User base dir: " $(python3 -c "import site; print(site.getuserbase())") | tee -a install.log.txt
echo "Site packages: " $(python3 -c "import site; print(site.getsitepackages())") | tee -a install.log.txt
echo "User site packages: " $(python3 -c "import site; print(site.getusersitepackages())") | tee -a install.log.txt

# Teste ob Pip3 installiert ist
info_section "Testing Pip3"
check_prg 'which pip3'
if ! $RETURN; then echo "Pip3 is not installed" | tee -a install.log.txt
  fehler
else
  echo "Pip3 is installed" | tee -a install.log.txt
  echo "Version: " $(pip3 -V) | tee -a install.log.txt
fi
RET=$(osascript -e 'display alert "Python3 and Pip3 are installed" message "Now the missing Python modules are determined and installed" buttons {"OK","Cancel"} default button 1')

#echo "* Updating Pip3" | tee -a install.log.txt
#pip3 install --user --upgrade pip
#if test "$RET" = "button returned:Cancel"; then exit; fi

# Teste und installiere fehlende Python-Module
info_section "Checking for missing Python modules"
echo "Required modules: "$PY_MODULE | tee -a install.log.txt
check_prg 'python3 -c "import '$PY_MODULE'"'
if ! $RETURN; then
    # Installiere fehlde Python Module
    echo "* Some modules are missing -> Start Installation" | tee -a install.log.txt
    pip3 install --user $PY_MODULE_PIP
    # Nochmal testen
    echo "* Checking for missing Python modules again" | tee -a install.log.txt
    echo "Required modules: "$PY_MODULE | tee -a install.log.txt
    check_prg 'python3 -c "import '$PY_MODULE'"'
    if ! $RETURN; then
      # Immer noch ein Problem -> Manuelle Installation
      echo "* Some dependencies are still not fulfilled" | tee -a install.log.txt
      fehler
    exit
fi
fi
echo "All modules are found" | tee -a install.log.txt

RET=$(osascript -e 'display alert "All Python3 modules are installed" message "Testing Postgres Tools now" buttons {"OK","Cancel"} default button 1')
if test "$RET" = "button returned:Cancel"; then exit; fi

# Optional: Teste Psql Commandline Tools (psql, pg_dump)
info_section "Testing _optional_ PostgreSQL tools: psql, pg_dump"
check_prg 'which psql pg_dump'
if ! $RETURN; then
    # PostgreSQL nicht installiert
    echo "PostgreSQL tools are missing" | tee -a install.log.txt
    RET=$(osascript -e 'display alert "PostgreSQL tools are missing" message "PostgreSQL tools are not found. Core database features of CRITTERBASE will not be available, e.g. backup and restore of whole databases. Please install the tools later manually, take a look at the documentation of the CRITTERBASE Collecors App" as critical buttons {"OK","Cancel"} default button 1')
    if test "$RET" = "button returned:Cancel"; then exit; fi
else
    # PostgreSQL found
     echo "PostgreSQL tools are found" | tee -a install.log.txt
    RET=$(osascript -e 'display alert "PostgreSQL tools found" message "PostgreSQL tools are found. Core database features of CRITTERBASE will be available, e.g. backup and restore of whole databases" buttons {"OK","Cancel"} default button 1')
    if test "$RET" = "button returned:Cancel"; then exit; fi
fi
RET=$(osascript -e 'display alert "Final testing" message "All dependencies will be tested again" buttons {"OK","Cancel"} default button 1')
if test "$RET" = "button returned:Cancel"; then exit; fi

# Abschlusstest
info_section "Testing all dependencies again"
echo "Required modules: "$PY_MODULE | tee -a install.log.txt
check_prg 'python3 -c "import '$PY_MODULE'"'
if $RETURN;
    then
        # Ok: Starte Programm
        echo "All dependencies are fulfilled" | tee -a install.log.txt
	    dialog_info "Everything is OK" "All dependencies are fulfilled. CRITTERBASE Collector App will be started now. If this fails, please start manually"
        open -a TextEdit install.log.txt
		open "../../../CRITTERBASE.app"
        exit
    else
        # Immer noch ein Problem -> Manuelle Installation
        echo "Still not all dependencies are fulfilled" | tee -a install.log.txt
        open -a TextEdit install.log.txt
        fehler
        exit
fi